#include "main.h"

#include <stdio.h>
#include <stdlib.h>

#include "lua.hpp"
#include "lauxlib.h"
#include "lualib.h"


const char* script_name = "script.lua";
const char* init_tablename = "init_parameters";

static const luaL_Reg loadedlibs[] = {
  {"_G", luaopen_base},
  {LUA_LOADLIBNAME, luaopen_package},
  /*{LUA_COLIBNAME, luaopen_coroutine},*/
  {LUA_TABLIBNAME, luaopen_table},
  /*{LUA_IOLIBNAME, luaopen_io},*/
  /*{LUA_OSLIBNAME, luaopen_os},*/
  {LUA_STRLIBNAME, luaopen_string},
  /*{LUA_BITLIBNAME, luaopen_bit32},*/
  {LUA_MATHLIBNAME, luaopen_math},
  /*{LUA_DBLIBNAME, luaopen_debug},*/
  {NULL, NULL}
};

void error_msg(lua_State *L)
{
  fprintf(stderr, "%s\n", lua_tostring(L, -1));   
  lua_pop(L, -1);
}

void openlibs(lua_State *L)
{
  const luaL_Reg *lib; 
  for (lib = loadedlibs; lib->func; lib++) {
    luaL_requiref(L, lib->name, lib->func, 1); /* like 'require' in Lua */
    lua_pop(L, 1);
  }
}

int setup_init_param(lua_State *L, init_param_t& init_param)
{
  lua_getglobal(L, init_tablename); /* push table into stack */
  if(!lua_istable(L, -1)) {
    error_msg(L);
    return 1;  
  }
  
  lua_getfield(L, -1, "speed"); /* get table["speed"] */
  if(!lua_isnumber(L, -1)) {
    error_msg(L);
    return 1;
  }  
  init_param.speed = lua_tonumber(L, -1);
  lua_pop(L, 1); /* pop 'speed' key from stack */

  lua_getfield(L, -1, "motion_func_name"); /* get table["motion_func_name"] */
  if(!lua_isstring(L, -1)) {
    error_msg(L);
    return 1;
  }
  
  init_param.motion_lua_func = lua_tostring(L, -1);
  lua_pop(L, 1); /* pop 'motion_func_name' key from stack */
  return 0;
}

int main (void) 
{
  lua_State *L = luaL_newstate();  /* create state */
  if (L == NULL) {
    fprintf(stderr, "%s\n", "cannot create state: not enough memory");
    return EXIT_FAILURE;
  }
  openlibs(L);
  
  int status = luaL_loadfile(L, script_name); 
  if(status != LUA_OK) {
    error_msg(L);
  }
  
  status = lua_pcall(L, 0, 0, 0); /* call in protected mode */
  if(status != LUA_OK) {
    error_msg(L); 
  }    
  
  // Setup init parameters by default
  init_param_t init_param;
  init_param.speed = 0;
  init_param.motion_lua_func = "none";  

  if(setup_init_param(L, init_param) != 0) {
    fprintf(stderr, "%s\n", "cannot initialize parameters");
    return EXIT_FAILURE; 
  }

  printf("speed: %d\n", init_param.speed);
  printf("motion_lua_func: %s\n", init_param.motion_lua_func.c_str());

  lua_getglobal(L, init_param.motion_lua_func.c_str());
  status = lua_pcall(L, 0, 1, 0); /* call in protected mode; 0 arg, 1 result */
  if(status != LUA_OK) {
    error_msg(L);
    return EXIT_FAILURE;
  }    

  int scan_res = lua_tonumber(L, -1); /* 1 result */
  printf("Analyse scanning result: %d\n", scan_res);

  lua_close(L);
  return EXIT_SUCCESS;
}